
/* Chrysalide - Outil d'analyse de fichiers binaires
 * formats.c - équivalent Python du fichier "core/formats.c"
 *
 * Copyright (C) 2015-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "formats.h"


#include <pygobject.h>


#include <core/formats.h>


#include "../helpers.h"



/* Fournit le nom humain du format binaire visé. */
static PyObject *py_formats_get_binary_format_name(PyObject *, PyObject *);

/* Définit les constantes pour les paramètres. */
static bool py_formats_define_constants(PyTypeObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = NULL car méthode statique.                            *
*                args = non utilisé ici.                                      *
*                                                                             *
*  Description : Fournit le nom humain du format binaire visé.                *
*                                                                             *
*  Retour      : Désignation humaine trouvée ou None.                         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_formats_get_binary_format_name(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Désignation à retourner     */
    const char *key;                        /* Nom court du format         */
    int ret;                                /* Bilan de lecture des args.  */
    const char *name;                       /* Désignation humaine         */

    ret = PyArg_ParseTuple(args, "s", &key);
    if (!ret) return NULL;

    name = get_binary_format_name(key);

    if (name != NULL)
        result = PyUnicode_FromString(name);

    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_formats_type(void)
{
    static PyMethodDef py_formats_methods[] = {

        { "get_binary_format_name", py_formats_get_binary_format_name,
          METH_VARARGS | METH_STATIC,
          "get_binary_format_name(key, /)\n--\n\nGive access to the main configuration of Chrysalide."
        },
        { NULL }

    };

    static PyTypeObject py_formats_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name = "pychrysalide.core.formats",
        .tp_basicsize = sizeof(PyObject),

        .tp_flags = Py_TPFLAGS_DEFAULT,

        .tp_doc = "Access to the code formats",

        .tp_methods = py_formats_methods

    };

    return &py_formats_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : obj_type = type dont le dictionnaire est à compléter.        *
*                                                                             *
*  Description : Définit les constantes pour les paramètres.                  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool py_formats_define_constants(PyTypeObject *obj_type)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    result &= PyDict_AddIntMacro(obj_type, FMS_MATCHED);
    result &= PyDict_AddIntMacro(obj_type, FMS_FORWARDED);
    result &= PyDict_AddIntMacro(obj_type, FMS_UNKNOWN);

    result &= PyDict_AddIntMacro(obj_type, FMS_COUNT);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.core.formats'.         *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_formats(PyObject *module)
{
    PyTypeObject *py_formats_type;          /* Type Python pour 'formats'  */
    int ret;                                /* Bilan d'un appel            */

    py_formats_type = get_python_formats_type();

    py_formats_type->tp_new = PyType_GenericNew;

    if (PyType_Ready(py_formats_type) != 0)
        return false;

    if (!py_formats_define_constants(py_formats_type))
        return false;

    Py_INCREF(py_formats_type);
    ret = PyModule_AddObject(module, "formats", (PyObject *)py_formats_type);

    return (ret == 0);

}
